/**********
Copyright 1991 Regents of the University of California. All rights reserved.
Authors :  1991 David Gates
**********/

/* Member of CIDER device simulator
 * Version: 1b1
 */

#ifndef ngspice_CARDDEFS_H
#define ngspice_CARDDEFS_H

/* Data Structures and Definitions for Device Simulation Cards */
#include "ngspice/bdrydefs.h"
#include "ngspice/contdefs.h"
#include "ngspice/domndefs.h"
#include "ngspice/dopdefs.h"
#include "ngspice/elctdefs.h"
#include "ngspice/meshdefs.h"
#include "ngspice/methdefs.h"
#include "ngspice/matldefs.h"
#include "ngspice/mobdefs.h"
#include "ngspice/modldefs.h"
#include "ngspice/optndefs.h"
#include "ngspice/outpdefs.h"

#endif
